#pragma once

void printA(void);
void printB(void);
void printC(void);
void printD(void);
void printE(void);
void printF(void);
void printG(void);
void printH(void);
void printI(void);
void printJ(void);
void printK(void);
